#include "animate.h"
#include "demo.h"

using namespace std;

namespace Msp {
namespace DemoScene {

Animate::Animate():
	target(0),
	anim(0),
	stacked(false),
	player(0)
{ }

Animate::Animate(GL::Placeable &t, const GL::Animation &a, GL::AnimationPlayer &p):
	target(&t),
	anim(&a),
	player(&p)
{ }

void Animate::validate() const
{
	if(!target)
		throw logic_error("null target");
	if(!anim)
		throw logic_error("null animation");
	if(!player)
		throw logic_error("null player");
}

void Animate::start(float, float d)
{
	float speed = 1.0f;
	if(!anim->is_looping() && d)
		speed = (anim->get_duration()/Time::sec)/d;
	if(target_obj)
	{
		if(stacked)
			player->play_stacked(*target_obj, *anim, speed);
		else
			player->play(*target_obj, *anim, speed);
	}
	else
	{
		if(stacked)
			player->play_stacked(*target, *anim, speed);
		else
			player->play(*target, *anim, speed);
	}
}


Animate::Loader::Loader(Animate &a, Demo &d):
	DataFile::DerivedObjectLoader<Animate, Action::Loader>(a, d)
{
	a.player = &demo.get_animation_player();
	add("animation", &Loader::animation);
	add("stacked", &Animate::stacked);
	add("target", &Loader::target);
}

void Animate::Loader::animation(const string &n)
{
	obj.anim = &demo.get_resources().get<GL::Animation>(n);
}

void Animate::Loader::target(const string &n)
{
	obj.target = &demo.get_thing<GL::Placeable>(n);
	obj.target_obj = dynamic_cast<GL::AnimatedObject *>(obj.target);
}

} // namespace DemoScene
} // namespace Msp
