#ifndef MSP_DEMOSCENE_LAUNCHER_H_
#define MSP_DEMOSCENE_LAUNCHER_H_

#include <msp/al/context.h>
#include <msp/al/device.h>
#include <msp/core/application.h>
#include <msp/graphics/display.h>
#include <msp/graphics/glcontext.h>
#include <msp/graphics/window.h>
#include <msp/input/keyboard.h>
#include <msp/io/base.h>
#include <msp/time/timedelta.h>
#include <msp/time/timestamp.h>
#include "resources.h"

namespace Msp {
namespace DemoScene {

class Demo;
class LaunchScreen;

class LauncherBase
{
protected:
	struct Options
	{
		Msp::Graphics::WindowOptions win_opts;
		Msp::Graphics::WindowOptions fullscreen_opts;
		int start_fullscreen;
		std::string frame_dump_fn;
		float framerate;
		Msp::Time::TimeDelta seek;
		bool no_music;
		bool no_vsync;

		Options(Msp::Graphics::Display &, int, char **);

		static void parse_size(const std::string &, Msp::Graphics::WindowOptions &);
	};

	Msp::Graphics::Display display;
	Options options;
	Msp::Graphics::Window window;
	Msp::Graphics::GLContext gl_context;
	Msp::Input::Keyboard keyboard;
	Msp::AL::Device *al_device;
	Msp::AL::Context *al_context;

	LaunchScreen *launch_screen;
	Msp::Time::TimeStamp timeout;

	Demo *demo;
	Msp::Time::TimeStamp next_frame;
	Msp::IO::Base *frame_dump;
	unsigned frame_size;
	char *frame_dump_buffer;

public:
	LauncherBase(int, char **);
	~LauncherBase();

protected:
	virtual Resources &get_resources() = 0;
	virtual Demo *create_demo() = 0;
	virtual void start();
	virtual void tick();
	void tick_launch_screen();
	virtual void tick_demo();

	void start_demo(bool);
	void key_press(unsigned);

	virtual void exit(int) = 0;
};

template<typename T>
class Launcher: public Msp::RegisteredApplication<T>, public LauncherBase
{
protected:
	Launcher(int argc, char **argv): LauncherBase(argc, argv) { }

	virtual int main() { start(); return Msp::Application::main(); }
	virtual void tick() { LauncherBase::tick(); }
	virtual void exit(int c) { Msp::Application::exit(c); }
};

} // namespace DemoScene
} // namespace Msp

#endif
